import { isVNode, createVNode, render, mergeProps, Fragment } from 'vue';
import '../../../../utils/index.mjs';
import { InAlert } from '../../alert/index.mjs';
import { InTypography } from '../../typography/index.mjs';
import Notistack from './notistack.vue.mjs';
import useCss from './index.jss.mjs';
import { generateId } from '../../../../utils/src/dom/id.mjs';
import { isEmpty } from '../../../../utils/src/javascript/check.mjs';
import { singleton } from '../../../../utils/src/javascript/singleton.mjs';

"use strict";
function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !isVNode(s);
}
class InMessageClass {
  constructor(props = {}) {
    this._rendered = false;
    this._body = document.body;
    this._notistack = createVNode(Notistack, null, null);
    this._defaultProps = {
      title: "",
      variant: "filled",
      severity: "info",
      icon: true,
      customIcon: void 0,
      action: void 0,
      actionEvent: (node) => {
        this._removeMessage(node.id);
      },
      content: "",
      duration: 2e3,
      transition: "all 0.5s ease",
      placement: {
        x: "right",
        y: "top"
      }
    };
    this._handleMouseenter = (timer) => {
      clearTimeout(timer);
      timer = null;
    };
    this._defaultProps = {
      ...this._defaultProps,
      ...props
    };
    const cssClass = useCss(this._defaultProps);
    this._notistack = createVNode(Notistack, {
      "class": cssClass
    }, null);
    this._render();
  }
  _render() {
    if (!this._rendered) {
      render(this._notistack, this._body);
      this._rendered = true;
    }
  }
  push(props = {}) {
    this._render();
    this._pushMessage(props);
  }
  _pushMessage(props = {}) {
    const {
      title,
      content,
      customIcon,
      action,
      actionEvent,
      duration,
      placement: _placement,
      transition: _transition,
      ...alertProps
    } = {
      ...this._defaultProps,
      ...props
    };
    let timer = null;
    const Alert = InAlert;
    const Icon = customIcon;
    const Action = action;
    const id = generateId();
    const alertVNode = createVNode(Alert, mergeProps(alertProps, {
      "onMouseenter": () => this._handleMouseenter(timer),
      "onMouseleave": () => this._startTimer(alertVNode, duration)
    }), {
      icon: customIcon ? () => createVNode(Icon, null, null) : void 0,
      default: () => createVNode(Fragment, null, [!isEmpty(title) && createVNode(InTypography, {
        "variant": "title.medium",
        "component": "h1",
        "color": "inherit"
      }, _isSlot(title) ? title : {
        default: () => [title]
      }), content]),
      action: action ? (icon) => createVNode(Action, mergeProps(mergeProps(icon, action.props), {
        "onClick": () => actionEvent(alertVNode, this._removeMessage.bind(this))
      }), null) : void 0
    });
    alertVNode.id = id;
    this._notistack.component?.exposed?.add(alertVNode);
    timer = this._startTimer(alertVNode, duration);
  }
  _removeMessage(condition) {
    this._notistack.component?.exposed?.remove(condition);
  }
  _startTimer(alertVNode, duration) {
    return setTimeout(() => {
      this._removeMessage(alertVNode.id);
    }, duration);
  }
  info(props = {}) {
    this.push({
      ...props,
      severity: "info"
    });
  }
  success(props = {}) {
    this.push({
      ...props,
      severity: "success"
    });
  }
  warning(props = {}) {
    this.push({
      ...props,
      severity: "warning"
    });
  }
  error(props = {}) {
    this.push({
      ...props,
      severity: "error"
    });
  }
  removeAll() {
    this._notistack.component?.exposed?.removeAll();
  }
  static install(app) {
    app.config.globalProperties.$message = singleton(InMessageClass);
  }
}
var InMessage = singleton(InMessageClass);

export { InMessageClass, InMessage as default };
//# sourceMappingURL=index.mjs.map
