'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _Stack = require('./_Stack.js');
var _equalArrays = require('./_equalArrays.js');
var _equalByTag = require('./_equalByTag.js');
var _equalObjects = require('./_equalObjects.js');
var _getTag = require('./_getTag.js');
var isArray = require('./isArray.js');
var isBuffer = require('./isBuffer.js');
var isTypedArray = require('./isTypedArray.js');

/** Used to compose bitmasks for value comparisons. */
var COMPARE_PARTIAL_FLAG = 1;

/** `Object#toString` result references. */
var argsTag = '[object Arguments]',
    arrayTag = '[object Array]',
    objectTag = '[object Object]';

/** Used for built-in method references. */
var objectProto = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty = objectProto.hasOwnProperty;

/**
 * A specialized version of `baseIsEqual` for arrays and objects which performs
 * deep comparisons and tracks traversed objects enabling objects with circular
 * references to be compared.
 *
 * @private
 * @param {Object} object The object to compare.
 * @param {Object} other The other object to compare.
 * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
 * @param {Function} customizer The function to customize comparisons.
 * @param {Function} equalFunc The function to determine equivalents of values.
 * @param {Object} [stack] Tracks traversed `object` and `other` objects.
 * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
 */
function baseIsEqualDeep(object, other, bitmask, customizer, equalFunc, stack) {
  var objIsArr = isArray.default(object),
      othIsArr = isArray.default(other),
      objTag = objIsArr ? arrayTag : _getTag.default(object),
      othTag = othIsArr ? arrayTag : _getTag.default(other);

  objTag = objTag == argsTag ? objectTag : objTag;
  othTag = othTag == argsTag ? objectTag : othTag;

  var objIsObj = objTag == objectTag,
      othIsObj = othTag == objectTag,
      isSameTag = objTag == othTag;

  if (isSameTag && isBuffer.default(object)) {
    if (!isBuffer.default(other)) {
      return false;
    }
    objIsArr = true;
    objIsObj = false;
  }
  if (isSameTag && !objIsObj) {
    stack || (stack = new _Stack.default);
    return (objIsArr || isTypedArray.default(object))
      ? _equalArrays.default(object, other, bitmask, customizer, equalFunc, stack)
      : _equalByTag.default(object, other, objTag, bitmask, customizer, equalFunc, stack);
  }
  if (!(bitmask & COMPARE_PARTIAL_FLAG)) {
    var objIsWrapped = objIsObj && hasOwnProperty.call(object, '__wrapped__'),
        othIsWrapped = othIsObj && hasOwnProperty.call(other, '__wrapped__');

    if (objIsWrapped || othIsWrapped) {
      var objUnwrapped = objIsWrapped ? object.value() : object,
          othUnwrapped = othIsWrapped ? other.value() : other;

      stack || (stack = new _Stack.default);
      return equalFunc(objUnwrapped, othUnwrapped, bitmask, customizer, stack);
    }
  }
  if (!isSameTag) {
    return false;
  }
  stack || (stack = new _Stack.default);
  return _equalObjects.default(object, other, bitmask, customizer, equalFunc, stack);
}

exports.default = baseIsEqualDeep;
//# sourceMappingURL=_baseIsEqualDeep.js.map
