'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _baseKeys = require('./_baseKeys.js');
var _getTag = require('./_getTag.js');
var isArguments = require('./isArguments.js');
var isArray = require('./isArray.js');
var isArrayLike = require('./isArrayLike.js');
var isBuffer = require('./isBuffer.js');
var _isPrototype = require('./_isPrototype.js');
var isTypedArray = require('./isTypedArray.js');

/** `Object#toString` result references. */
var mapTag = '[object Map]',
    setTag = '[object Set]';

/** Used for built-in method references. */
var objectProto = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty = objectProto.hasOwnProperty;

/**
 * Checks if `value` is an empty object, collection, map, or set.
 *
 * Objects are considered empty if they have no own enumerable string keyed
 * properties.
 *
 * Array-like values such as `arguments` objects, arrays, buffers, strings, or
 * jQuery-like collections are considered empty if they have a `length` of `0`.
 * Similarly, maps and sets are considered empty if they have a `size` of `0`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is empty, else `false`.
 * @example
 *
 * _.isEmpty(null);
 * // => true
 *
 * _.isEmpty(true);
 * // => true
 *
 * _.isEmpty(1);
 * // => true
 *
 * _.isEmpty([1, 2, 3]);
 * // => false
 *
 * _.isEmpty({ 'a': 1 });
 * // => false
 */
function isEmpty(value) {
  if (value == null) {
    return true;
  }
  if (isArrayLike.default(value) &&
      (isArray.default(value) || typeof value == 'string' || typeof value.splice == 'function' ||
        isBuffer.default(value) || isTypedArray.default(value) || isArguments.default(value))) {
    return !value.length;
  }
  var tag = _getTag.default(value);
  if (tag == mapTag || tag == setTag) {
    return !value.size;
  }
  if (_isPrototype.default(value)) {
    return !_baseKeys.default(value).length;
  }
  for (var key in value) {
    if (hasOwnProperty.call(value, key)) {
      return false;
    }
  }
  return true;
}

exports.default = isEmpty;
//# sourceMappingURL=isEmpty.js.map
