'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _arrayEach = require('./_arrayEach.js');
var _arrayPush = require('./_arrayPush.js');
var _baseFunctions = require('./_baseFunctions.js');
var _copyArray = require('./_copyArray.js');
var isFunction = require('./isFunction.js');
var isObject = require('./isObject.js');
var keys = require('./keys.js');

/**
 * Adds all own enumerable string keyed function properties of a source
 * object to the destination object. If `object` is a function, then methods
 * are added to its prototype as well.
 *
 * **Note:** Use `_.runInContext` to create a pristine `lodash` function to
 * avoid conflicts caused by modifying the original.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Util
 * @param {Function|Object} [object=lodash] The destination object.
 * @param {Object} source The object of functions to add.
 * @param {Object} [options={}] The options object.
 * @param {boolean} [options.chain=true] Specify whether mixins are chainable.
 * @returns {Function|Object} Returns `object`.
 * @example
 *
 * function vowels(string) {
 *   return _.filter(string, function(v) {
 *     return /[aeiou]/i.test(v);
 *   });
 * }
 *
 * _.mixin({ 'vowels': vowels });
 * _.vowels('fred');
 * // => ['e']
 *
 * _('fred').vowels().value();
 * // => ['e']
 *
 * _.mixin({ 'vowels': vowels }, { 'chain': false });
 * _('fred').vowels();
 * // => ['e']
 */
function mixin(object, source, options) {
  var props = keys.default(source),
      methodNames = _baseFunctions.default(source, props);

  var chain = !(isObject.default(options) && 'chain' in options) || !!options.chain,
      isFunc = isFunction.default(object);

  _arrayEach.default(methodNames, function(methodName) {
    var func = source[methodName];
    object[methodName] = func;
    if (isFunc) {
      object.prototype[methodName] = function() {
        var chainAll = this.__chain__;
        if (chain || chainAll) {
          var result = object(this.__wrapped__),
              actions = result.__actions__ = _copyArray.default(this.__actions__);

          actions.push({ 'func': func, 'args': arguments, 'thisArg': object });
          result.__chain__ = chainAll;
          return result;
        }
        return func.apply(object, _arrayPush.default([this.value()], arguments));
      };
    }
  });

  return object;
}

exports.default = mixin;
//# sourceMappingURL=mixin.js.map
