'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _arrayMap = require('./_arrayMap.js');
var _baseClone = require('./_baseClone.js');
var _baseUnset = require('./_baseUnset.js');
var _castPath = require('./_castPath.js');
var _copyObject = require('./_copyObject.js');
var _customOmitClone = require('./_customOmitClone.js');
var _flatRest = require('./_flatRest.js');
var _getAllKeysIn = require('./_getAllKeysIn.js');

/** Used to compose bitmasks for cloning. */
var CLONE_DEEP_FLAG = 1,
    CLONE_FLAT_FLAG = 2,
    CLONE_SYMBOLS_FLAG = 4;

/**
 * The opposite of `_.pick`; this method creates an object composed of the
 * own and inherited enumerable property paths of `object` that are not omitted.
 *
 * **Note:** This method is considerably slower than `_.pick`.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Object
 * @param {Object} object The source object.
 * @param {...(string|string[])} [paths] The property paths to omit.
 * @returns {Object} Returns the new object.
 * @example
 *
 * var object = { 'a': 1, 'b': '2', 'c': 3 };
 *
 * _.omit(object, ['a', 'c']);
 * // => { 'b': '2' }
 */
var omit = _flatRest.default(function(object, paths) {
  var result = {};
  if (object == null) {
    return result;
  }
  var isDeep = false;
  paths = _arrayMap.default(paths, function(path) {
    path = _castPath.default(path, object);
    isDeep || (isDeep = path.length > 1);
    return path;
  });
  _copyObject.default(object, _getAllKeysIn.default(object), result);
  if (isDeep) {
    result = _baseClone.default(result, CLONE_DEEP_FLAG | CLONE_FLAT_FLAG | CLONE_SYMBOLS_FLAG, _customOmitClone.default);
  }
  var length = paths.length;
  while (length--) {
    _baseUnset.default(result, paths[length]);
  }
  return result;
});

exports.default = omit;
//# sourceMappingURL=omit.js.map
