'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _arrayEach = require('./_arrayEach.js');
var _baseCreate = require('./_baseCreate.js');
var _baseForOwn = require('./_baseForOwn.js');
var _baseIteratee = require('./_baseIteratee.js');
var _getPrototype = require('./_getPrototype.js');
var isArray = require('./isArray.js');
var isBuffer = require('./isBuffer.js');
var isFunction = require('./isFunction.js');
var isObject = require('./isObject.js');
var isTypedArray = require('./isTypedArray.js');

/**
 * An alternative to `_.reduce`; this method transforms `object` to a new
 * `accumulator` object which is the result of running each of its own
 * enumerable string keyed properties thru `iteratee`, with each invocation
 * potentially mutating the `accumulator` object. If `accumulator` is not
 * provided, a new object with the same `[[Prototype]]` will be used. The
 * iteratee is invoked with four arguments: (accumulator, value, key, object).
 * Iteratee functions may exit iteration early by explicitly returning `false`.
 *
 * @static
 * @memberOf _
 * @since 1.3.0
 * @category Object
 * @param {Object} object The object to iterate over.
 * @param {Function} [iteratee=_.identity] The function invoked per iteration.
 * @param {*} [accumulator] The custom accumulator value.
 * @returns {*} Returns the accumulated value.
 * @example
 *
 * _.transform([2, 3, 4], function(result, n) {
 *   result.push(n *= n);
 *   return n % 2 == 0;
 * }, []);
 * // => [4, 9]
 *
 * _.transform({ 'a': 1, 'b': 2, 'c': 1 }, function(result, value, key) {
 *   (result[value] || (result[value] = [])).push(key);
 * }, {});
 * // => { '1': ['a', 'c'], '2': ['b'] }
 */
function transform(object, iteratee, accumulator) {
  var isArr = isArray.default(object),
      isArrLike = isArr || isBuffer.default(object) || isTypedArray.default(object);

  iteratee = _baseIteratee.default(iteratee, 4);
  if (accumulator == null) {
    var Ctor = object && object.constructor;
    if (isArrLike) {
      accumulator = isArr ? new Ctor : [];
    }
    else if (isObject.default(object)) {
      accumulator = isFunction.default(Ctor) ? _baseCreate.default(_getPrototype.default(object)) : {};
    }
    else {
      accumulator = {};
    }
  }
  (isArrLike ? _arrayEach.default : _baseForOwn.default)(object, function(value, index, object) {
    return iteratee(accumulator, value, index, object);
  });
  return accumulator;
}

exports.default = transform;
//# sourceMappingURL=transform.js.map
